package com.franckbarbier._Programmable_thermostat_MODEL._Programmable_thermostat;

import com.franckbarbier._Programmable_thermostat_MODEL._Relays.*;
import com.franckbarbier._Programmable_thermostat_MODEL._Switchs.*;
import com.franckbarbier._Temperature.*;

final class Program {

    Temperature target_temperature;
    java.util.Calendar time;
    long difference;
}

public class Programmable_thermostat extends com.pauware.pauware_engine.Core.AbstractTimer_monitor implements Programmable_thermostat_input, Fan_switch_output, Run_indicator_output, Season_switch_output, Temperature_sensor_output, com.pauware.pauware_engine.Core.Manageable_base {

    /**
     * UML control flow
     */
    protected com.franckbarbier._Programmable_thermostat_GUI.Programmable_thermostat_output _user_interface;

    public void add_user_interface(com.franckbarbier._Programmable_thermostat_GUI.Programmable_thermostat_output user_interface) {
        _user_interface = user_interface;
    }
    /**
     * UML attributes
     */
    public static final int From_Setup_to_Operate_delay = 90; // 90 sec.

    protected Temperature _ambient_temperature;
    protected final float _delta = .1F;
    protected static Temperature _Max;

    public static float Max(byte temperature_unit) {
        switch (temperature_unit) {
            case Temperature.Celsius:
                return _Max.asCelsius();
            case Temperature.Fahrenheit:
                return _Max.asFahrenheit();
            case Temperature.Kelvin:
                return _Max.asKelvin();
            default:
                return _Max.asFahrenheit();
        }
    }
    protected static Temperature _Min;

    public static float Min(byte temperature_unit) {
        switch (temperature_unit) {
            case Temperature.Celsius:
                return _Min.asCelsius();
            case Temperature.Fahrenheit:
                return _Min.asFahrenheit();
            case Temperature.Kelvin:
                return _Min.asKelvin();
            default:
                return _Min.asFahrenheit();
        }
    }
    public static final float Target_temperature_precision = (float) 5E-1;
    protected Temperature _target_temperature;
    public static final byte Default_temperature_mode = Temperature.Celsius;
    protected byte _temperature_mode = Default_temperature_mode;
    protected java.util.Calendar _time = java.util.Calendar.getInstance();
    /**
     * UML associations
     */
    protected Program[] _program = new Program[8];

    protected Air_conditioner_relay _air_conditioner_relay;
    protected Fan_relay _fan_relay;
    protected Furnace_relay _furnace_relay;
    protected Fan_switch _fan_switch;
    protected Season_switch _season_switch;
    protected Run_indicator _run_indicator;
    /**
     * UML state machine diagram
     */
    protected com.pauware.pauware_engine.Core.AbstractState _Run;
    protected com.pauware.pauware_engine.Core.AbstractState _Hold;
    protected com.pauware.pauware_engine.Core.AbstractState _Current_date_and_time_displaying;
    protected com.pauware.pauware_engine.Core.AbstractState _Ambient_temperature_displaying;
    protected com.pauware.pauware_engine.Core.AbstractState _Target_temperature_displaying;
    protected com.pauware.pauware_engine.Core.AbstractState _Operate;
    protected com.pauware.pauware_engine.Core.AbstractState _Set_current_minute;
    protected com.pauware.pauware_engine.Core.AbstractState _Set_current_hour;
    protected com.pauware.pauware_engine.Core.AbstractState _Set_time;
    protected com.pauware.pauware_engine.Core.AbstractState _Set_current_day;
    protected com.pauware.pauware_engine.Core.AbstractState _Set_period;
    protected com.pauware.pauware_engine.Core.AbstractState _Set_program_time;
    protected com.pauware.pauware_engine.Core.AbstractState _Set_program_target_temperature;
    protected com.pauware.pauware_engine.Core.AbstractState _Set_program;
    protected com.pauware.pauware_engine.Core.AbstractState _Current_date_and_time_refreshing;
    protected com.pauware.pauware_engine.Core.AbstractState _Program_target_temperature_refreshing;
    protected com.pauware.pauware_engine.Core.AbstractState _Period_and_program_time_refreshing;
    protected com.pauware.pauware_engine.Core.AbstractState _Program_refreshing;
    protected com.pauware.pauware_engine.Core.AbstractState _Setup;
    protected com.pauware.pauware_engine.Core.AbstractState _Control;
    protected com.pauware.pauware_engine.Core.AbstractStateMachine _Programmable_thermostat_state_machine;
    /**
     * UML state machine diagram variables
     */
    protected int _alternately = 0;
    protected int _no_input = 0;
    protected int _period = 1;

    /**
     * UML creation
     */
    static {
        try {
            _Max = new Temperature(28.F, Temperature.Celsius);
            _Min = new Temperature(14.F, Temperature.Celsius);
        } catch (Exception e) {
            System.exit(1);
        }
    }

    private void init_structure(Temperature temperature) throws com.pauware.pauware_engine.Exceptions.State_exception {
        try {
            _ambient_temperature = (Temperature) temperature.clone();
            _target_temperature = new Temperature((Max(Temperature.Celsius) + Min(Temperature.Celsius)) / 2.F, Temperature.Celsius, Target_temperature_precision);
        } catch (Exception e) {
            System.exit(1);
        }
        _air_conditioner_relay = new Air_conditioner_relay(temperature);
        _fan_relay = new Fan_relay();
        _fan_switch = new Fan_switch(this);
        _furnace_relay = new Furnace_relay(temperature);
        _run_indicator = new Run_indicator(this);
        _season_switch = new Season_switch(this);
        for (int i = 0; i < _program.length; i++) {
            _program[i] = new Program();
            _program[i].time = java.util.Calendar.getInstance();
            try {
                _program[i].target_temperature = new Temperature((Max(Temperature.Celsius) + Min(Temperature.Celsius)) / 2.F, Temperature.Celsius, Target_temperature_precision);
            } catch (Exception e) {
                System.exit(1);
            }
        }
    }

    private void init_behavior() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Run = new com.pauware.pauware_engine.Core.State("Run");
        _Hold = new com.pauware.pauware_engine.Core.State("Hold");
        _Hold.inputState();
        _Current_date_and_time_displaying = (new com.pauware.pauware_engine.Core.State("Current date and time displaying")).set_entryAction(this, "display_current_date_and_time");
        _Current_date_and_time_displaying.inputState();
        _Ambient_temperature_displaying = (new com.pauware.pauware_engine.Core.State("Ambient temperature displaying")).set_entryAction(this, "display_ambient_temperature");
        _Target_temperature_displaying = (new com.pauware.pauware_engine.Core.State("Target temperature displaying")).set_entryAction(this, "display_target_temperature");
        _Operate = (((_Run.xor(_Hold)).and(_Current_date_and_time_displaying.xor(_Ambient_temperature_displaying)).and(_Target_temperature_displaying)).name("Operate"));
        _Operate.inputState();
        _Set_current_minute = new com.pauware.pauware_engine.Core.State("Set current minute");
        _Set_current_hour = new com.pauware.pauware_engine.Core.State("Set current hour");
        _Set_time = (_Set_current_minute.xor(_Set_current_hour)).name("Set time");
        _Set_current_day = new com.pauware.pauware_engine.Core.State("Set current day");
        _Set_period = new com.pauware.pauware_engine.Core.State("Set period");
        _Set_program_time = new com.pauware.pauware_engine.Core.State("Set program time");
        _Set_program_target_temperature = new com.pauware.pauware_engine.Core.State("Set program target temperature");
        _Set_program = (_Set_period.and(_Set_program_time).and(_Set_program_target_temperature)).name("Set program");
        _Current_date_and_time_refreshing = (new com.pauware.pauware_engine.Core.State("Current date and time refreshing")).set_entryAction(this, "display_current_date_and_time");
        _Program_target_temperature_refreshing = (new com.pauware.pauware_engine.Core.State("Program target temperature refreshing")).set_entryAction(this, "display_program_target_temperature");
        _Period_and_program_time_refreshing = (new com.pauware.pauware_engine.Core.State("Period and program time refreshing")).set_entryAction(this, "display_period_and_program_time");
        _Program_refreshing = (_Program_target_temperature_refreshing.and(_Period_and_program_time_refreshing)).name("Program refreshing");
        _Setup = ((_Set_time.xor(_Set_current_day).xor(_Set_program).and(_Current_date_and_time_refreshing.xor(_Program_refreshing))).name("Setup"));
        _Control = new com.pauware.pauware_engine.Core.State("Control");
        _Programmable_thermostat_state_machine = new com.pauware.pauware_engine.Core.StateMachine((_Operate.xor(_Setup)).and(_Control), "Programmable thermostat", com.pauware.pauware_engine.Core.AbstractStateMachine.Don_t_show_on_system_out, new com.pauware.pauware2web.PauWare2Web_client());
        _Programmable_thermostat_state_machine.set_entryAction(this, "to_be_set", new Object[]{Long.valueOf(1000L)});
        _Programmable_thermostat_state_machine.set_exitAction(this, "to_be_killed");
    }

    public void start() throws com.pauware.pauware_engine.Exceptions.State_exception {
//        _Programmable_thermostat_state_machine.fires("ambient_temperature_changed", _Control, _Control, this, "season_switch_in_Is_cool_and_ambient_temperature_greaterThan_target_temperature", _air_conditioner_relay, "run");
        _Control.allowedEvent("ambient_temperature_changed", this, "season_switch_in_Is_cool_and_ambient_temperature_greaterThan_target_temperature", _air_conditioner_relay, "run");
//        _Programmable_thermostat_state_machine.fires("ambient_temperature_changed", _Control, _Control, this, "season_switch_in_Is_cool_and_ambient_temperature_greaterThan_target_temperature", _fan_relay, "run");
        _Control.allowedEvent("ambient_temperature_changed", this, "season_switch_in_Is_cool_and_ambient_temperature_greaterThan_target_temperature", _fan_relay, "run");
//        _Programmable_thermostat_state_machine.fires("ambient_temperature_changed", _Control, _Control, this, "season_switch_in_Is_cool_and_ambient_temperature_greaterThan_target_temperature", _run_indicator, "on");
        _Control.allowedEvent("ambient_temperature_changed", this, "season_switch_in_Is_cool_and_ambient_temperature_greaterThan_target_temperature", _run_indicator, "on");
//        _Programmable_thermostat_state_machine.fires("ambient_temperature_changed", _Control, _Control, this, "season_switch_in_Is_cool_and_ambient_temperature_lessThan_target_temperature_minus_delta", _air_conditioner_relay, "stop");
        _Control.allowedEvent("ambient_temperature_changed", this, "season_switch_in_Is_cool_and_ambient_temperature_lessThan_target_temperature_minus_delta", _air_conditioner_relay, "stop");
//        _Programmable_thermostat_state_machine.fires("ambient_temperature_changed", _Control, _Control, this, "season_switch_in_Is_cool_and_ambient_temperature_lessThan_target_temperature_minus_delta", _run_indicator, "off");
        _Control.allowedEvent("ambient_temperature_changed", this, "season_switch_in_Is_cool_and_ambient_temperature_lessThan_target_temperature_minus_delta", _run_indicator, "off");
//        _Programmable_thermostat_state_machine.fires("ambient_temperature_changed", _Control, _Control, this, "season_switch_in_Is_heat_and_ambient_temperature_lessThan_target_temperature", _furnace_relay, "run");
        _Control.allowedEvent("ambient_temperature_changed", this, "season_switch_in_Is_heat_and_ambient_temperature_lessThan_target_temperature", _furnace_relay, "run");
//        _Programmable_thermostat_state_machine.fires("ambient_temperature_changed", _Control, _Control, this, "season_switch_in_Is_heat_and_ambient_temperature_lessThan_target_temperature", _fan_relay, "run");
        _Control.allowedEvent("ambient_temperature_changed", this, "season_switch_in_Is_heat_and_ambient_temperature_lessThan_target_temperature", _fan_relay, "run");
//        _Programmable_thermostat_state_machine.fires("ambient_temperature_changed", _Control, _Control, this, "season_switch_in_Is_heat_and_ambient_temperature_lessThan_target_temperature", _run_indicator, "on");
        _Control.allowedEvent("ambient_temperature_changed", this, "season_switch_in_Is_heat_and_ambient_temperature_lessThan_target_temperature", _run_indicator, "on");
//        _Programmable_thermostat_state_machine.fires("ambient_temperature_changed", _Control, _Control, this, "season_switch_in_Is_heat_and_ambient_temperature_greaterThan_target_temperature_plus_delta", _furnace_relay, "stop");
        _Control.allowedEvent("ambient_temperature_changed", this, "season_switch_in_Is_heat_and_ambient_temperature_greaterThan_target_temperature_plus_delta", _furnace_relay, "stop");
//        _Programmable_thermostat_state_machine.fires("ambient_temperature_changed", _Control, _Control, this, "season_switch_in_Is_heat_and_ambient_temperature_greaterThan_target_temperature_plus_delta", _run_indicator, "off");
        _Control.allowedEvent("ambient_temperature_changed", this, "season_switch_in_Is_heat_and_ambient_temperature_greaterThan_target_temperature_plus_delta", _run_indicator, "off");

//        /*DYNAMICAL*/ _Programmable_thermostat_state_machine.fires("ambient_temperature_changed", _Control, _Control, this, "season_switch_in_Is_cool_and_ambient_temperature_lessThan_target_temperature_minus_delta", _fan_relay, "stop", new Object[]{Boolean.valueOf(_fan_switch.in("Is auto"))});
        /*DYNAMICAL*/ _Control.allowedEvent("ambient_temperature_changed", this, "season_switch_in_Is_cool_and_ambient_temperature_lessThan_target_temperature_minus_delta", _fan_relay, "stop", new Object[]{Boolean.valueOf(_fan_switch.in("Is auto"))});
//        /*DYNAMICAL*/ _Programmable_thermostat_state_machine.fires("ambient_temperature_changed", _Control, _Control, this, "season_switch_in_Is_heat_and_ambient_temperature_greaterThan_target_temperature_plus_delta", _fan_relay, "stop", new Object[]{Boolean.valueOf(_fan_switch.in("Is auto"))});
        /*DYNAMICAL*/ _Control.allowedEvent("ambient_temperature_changed", this, "season_switch_in_Is_heat_and_ambient_temperature_greaterThan_target_temperature_plus_delta", _fan_relay, "stop", new Object[]{Boolean.valueOf(_fan_switch.in("Is auto"))});
//        _Programmable_thermostat_state_machine.fires("fan_switch_turned_on", _Control, _Control, _fan_relay, "run");
        _Control.allowedEvent("fan_switch_turned_on", _fan_relay, "run");

        _Programmable_thermostat_state_machine.fires("f_c", _Ambient_temperature_displaying, _Ambient_temperature_displaying);
        _Programmable_thermostat_state_machine.fires("f_c", _Target_temperature_displaying, _Target_temperature_displaying, true, this, "switch_mode");
        _Programmable_thermostat_state_machine.fires("f_c", _Program_target_temperature_refreshing, _Program_target_temperature_refreshing, true, this, "switch_mode");
        _Programmable_thermostat_state_machine.fires("hold_temp", _Run, _Hold);
        _Programmable_thermostat_state_machine.fires("run_program", _Programmable_thermostat_state_machine, _Current_date_and_time_displaying);
        _Programmable_thermostat_state_machine.fires("run_program", _Programmable_thermostat_state_machine, _Run);

//        _Programmable_thermostat_state_machine.fires("season_switch_turned_off", _Control, _Control, true, _air_conditioner_relay, "stop");
        _Control.allowedEvent("season_switch_turned_off", _air_conditioner_relay, "stop");
//        _Programmable_thermostat_state_machine.fires("season_switch_turned_off", _Control, _Control, true, _furnace_relay, "stop");
        _Control.allowedEvent("season_switch_turned_off", _furnace_relay, "stop");
//        _Programmable_thermostat_state_machine.fires("season_switch_turned_off", _Control, _Control, true, _run_indicator, "off");
        _Control.allowedEvent("season_switch_turned_off", _run_indicator, "off");
//        /*DYNAMICAL*/ _Programmable_thermostat_state_machine.fires("season_switch_turned_off", _Control, _Control, true, _fan_relay, "stop", new Object[]{Boolean.valueOf(_fan_switch.in("Is auto"))});
        /*DYNAMICAL*/ _Control.allowedEvent("season_switch_turned_off", _fan_relay, "stop", new Object[]{Boolean.valueOf(_fan_switch.in("Is auto"))});

        _Programmable_thermostat_state_machine.fires("set_clock", _Programmable_thermostat_state_machine, _Current_date_and_time_refreshing);
        // Each time a setup-based key is pressed, one resets the '_no_input' variable to zero so that, after 90 sec., one goes back to 'Operate':
        _Programmable_thermostat_state_machine.fires("set_clock", _Programmable_thermostat_state_machine, _Set_current_minute, true, this, "set_no_input", new Object[]{Integer.valueOf(0)});
        _Programmable_thermostat_state_machine.fires("set_clock", _Set_current_minute, _Set_current_hour, true, this, "set_no_input", new Object[]{Integer.valueOf(0)});
        _Programmable_thermostat_state_machine.fires("set_day", _Programmable_thermostat_state_machine, _Set_current_day);
        // Each time a setup-based key is pressed, one resets the '_no_input' variable to zero so that, after 90 sec., one goes back to 'Operate':
        _Programmable_thermostat_state_machine.fires("set_day", _Programmable_thermostat_state_machine, _Current_date_and_time_refreshing, true, this, "set_no_input", new Object[]{Integer.valueOf(0)});

//        _Programmable_thermostat_state_machine.fires("target_temperature_changed", _Control, _Control, this, "season_switch_in_Is_cool_and_ambient_temperature_greaterThan_target_temperature", _air_conditioner_relay, "run", null);
        _Control.allowedEvent("target_temperature_changed", this, "season_switch_in_Is_cool_and_ambient_temperature_greaterThan_target_temperature", _air_conditioner_relay, "run");
//        _Programmable_thermostat_state_machine.fires("target_temperature_changed", _Control, _Control, this, "season_switch_in_Is_cool_and_ambient_temperature_greaterThan_target_temperature", _fan_relay, "run", null);
        _Control.allowedEvent("target_temperature_changed", this, "season_switch_in_Is_cool_and_ambient_temperature_greaterThan_target_temperature", _fan_relay, "run");
//        _Programmable_thermostat_state_machine.fires("target_temperature_changed", _Control, _Control, this, "season_switch_in_Is_cool_and_ambient_temperature_greaterThan_target_temperature", _run_indicator, "on", null);
        _Control.allowedEvent("target_temperature_changed", this, "season_switch_in_Is_cool_and_ambient_temperature_greaterThan_target_temperature", _run_indicator, "on");
//        _Programmable_thermostat_state_machine.fires("target_temperature_changed", _Control, _Control, this, "season_switch_in_Is_cool_and_ambient_temperature_lessThan_target_temperature_minus_delta", _air_conditioner_relay, "stop");
        _Control.allowedEvent("target_temperature_changed", this, "season_switch_in_Is_cool_and_ambient_temperature_lessThan_target_temperature_minus_delta", _air_conditioner_relay, "stop");
//        _Programmable_thermostat_state_machine.fires("target_temperature_changed", _Control, _Control, this, "season_switch_in_Is_cool_and_ambient_temperature_lessThan_target_temperature_minus_delta", _run_indicator, "off");
        _Control.allowedEvent("target_temperature_changed", this, "season_switch_in_Is_cool_and_ambient_temperature_lessThan_target_temperature_minus_delta", _run_indicator, "off");
//        _Programmable_thermostat_state_machine.fires("target_temperature_changed", _Control, _Control, this, "season_switch_in_Is_heat_and_ambient_temperature_lessThan_target_temperature", _furnace_relay, "run");
        _Control.allowedEvent("target_temperature_changed", this, "season_switch_in_Is_heat_and_ambient_temperature_lessThan_target_temperature", _furnace_relay, "run");
//        _Programmable_thermostat_state_machine.fires("target_temperature_changed", _Control, _Control, this, "season_switch_in_Is_heat_and_ambient_temperature_lessThan_target_temperature", _fan_relay, "run");
        _Control.allowedEvent("target_temperature_changed", this, "season_switch_in_Is_heat_and_ambient_temperature_lessThan_target_temperature", _fan_relay, "run");
//        _Programmable_thermostat_state_machine.fires("target_temperature_changed", _Control, _Control, this, "season_switch_in_Is_heat_and_ambient_temperature_lessThan_target_temperature", _run_indicator, "on");
        _Control.allowedEvent("target_temperature_changed", this, "season_switch_in_Is_heat_and_ambient_temperature_lessThan_target_temperature", _run_indicator, "on");
//        _Programmable_thermostat_state_machine.fires("target_temperature_changed", _Control, _Control, this, "season_switch_in_Is_heat_and_ambient_temperature_greaterThan_target_temperature_plus_delta", _furnace_relay, "stop");
        _Control.allowedEvent("target_temperature_changed", this, "season_switch_in_Is_heat_and_ambient_temperature_greaterThan_target_temperature_plus_delta", _furnace_relay, "stop");
//        _Programmable_thermostat_state_machine.fires("target_temperature_changed", _Control, _Control, this, "season_switch_in_Is_heat_and_ambient_temperature_greaterThan_target_temperature_plus_delta", _run_indicator, "off", null);
        _Control.allowedEvent("target_temperature_changed", this, "season_switch_in_Is_heat_and_ambient_temperature_greaterThan_target_temperature_plus_delta", _run_indicator, "off");
        /*DYNAMICAL*/ _Control.allowedEvent("target_temperature_changed", this, "season_switch_in_Is_cool_and_ambient_temperature_lessThan_target_temperature_minus_delta", _fan_relay, "stop", new Object[]{Boolean.valueOf(_fan_switch.in("Is auto"))});
        /*DYNAMICAL*/ _Control.allowedEvent("target_temperature_changed", this, "season_switch_in_Is_heat_and_ambient_temperature_greaterThan_target_temperature_plus_delta", _fan_relay, "stop", new Object[]{Boolean.valueOf(_fan_switch.in("Is auto"))});

// Each time a setup-based key is pressed, one resets the '_no_input' variable to zero so that, after 90 sec., one goes back to 'Operate':
//        _Programmable_thermostat_state_machine.fires("temp_down", _Program_target_temperature_refreshing, _Program_target_temperature_refreshing, true, this, "set_no_input", new Object[]{Integer.valueOf(0)});
        _Program_target_temperature_refreshing.allowedEvent("temp_down", this, "set_no_input", new Object[]{Integer.valueOf(0)});
        _Programmable_thermostat_state_machine.fires("temp_down", _Target_temperature_displaying, _Target_temperature_displaying, this, "target_temperature_greaterThan_Min", null, this, "target_temperature_changed", null, com.pauware.pauware_engine.Core.AbstractState.Reentrance);
//        /*DYNAMICAL*/ _Programmable_thermostat_state_machine.fires("temp_down", _Set_program_target_temperature, _Set_program_target_temperature, this, "program_period_target_temperature_greaterThan_Min", _program[_period - 1].target_temperature, "decrement");
        /*DYNAMICAL*/ _Set_program_target_temperature.allowedEvent("temp_down", this, "program_period_target_temperature_greaterThan_Min", _program[_period - 1].target_temperature, "decrement");
//        _Programmable_thermostat_state_machine.fires("temp_down", _Run, _Run, this, "target_temperature_greaterThan_Min", null, _target_temperature, "decrement");
        _Run.allowedEvent("temp_down", this, "target_temperature_greaterThan_Min", null, _target_temperature, "decrement");
//        _Programmable_thermostat_state_machine.fires("temp_down", _Hold, _Hold, this, "target_temperature_greaterThan_Min", null, _target_temperature, "decrement");
        _Hold.allowedEvent("temp_down", this, "target_temperature_greaterThan_Min", null, _target_temperature, "decrement");

// Each time a setup-based key is pressed, one resets the '_no_input' variable to zero so that, after 90 sec., one goes back to 'Operate':
//        _Programmable_thermostat_state_machine.fires("temp_up", _Program_target_temperature_refreshing, _Program_target_temperature_refreshing, true, this, "set_no_input", new Object[]{Integer.valueOf(0)});
        _Program_target_temperature_refreshing.allowedEvent("temp_up", this, "set_no_input", new Object[]{Integer.valueOf(0)});
        _Programmable_thermostat_state_machine.fires("temp_up", _Target_temperature_displaying, _Target_temperature_displaying, this, "target_temperature_lessThan_Max", null, this, "target_temperature_changed", null, com.pauware.pauware_engine.Core.AbstractState.Reentrance);
//        /*DYNAMICAL*/ _Programmable_thermostat_state_machine.fires("temp_up", _Set_program_target_temperature, _Set_program_target_temperature, this, "program_period_target_temperature_lessThan_Max", _program[_period - 1].target_temperature, "increment");
        /*DYNAMICAL*/ _Set_program_target_temperature.allowedEvent("temp_up", this, "program_period_target_temperature_lessThan_Max", _program[_period - 1].target_temperature, "increment");
//        _Programmable_thermostat_state_machine.fires("temp_up", _Run, _Run, this, "target_temperature_lessThan_Max", null, _target_temperature, "increment");
        _Run.allowedEvent("temp_up", this, "target_temperature_lessThan_Max", null, _target_temperature, "increment");
//        _Programmable_thermostat_state_machine.fires("temp_up", _Hold, _Hold, this, "target_temperature_lessThan_Max", null, _target_temperature, "increment");
        _Hold.allowedEvent("temp_up", this, "target_temperature_lessThan_Max", null, _target_temperature, "increment");

//        _Programmable_thermostat_state_machine.fires("time_backward", _Set_current_minute, _Set_current_minute, true, this, "set_time", new Object[]{Integer.valueOf(-60)});
        _Set_current_minute.allowedEvent("time_backward", this, "set_time", new Object[]{Integer.valueOf(-60)});
//        _Programmable_thermostat_state_machine.fires("time_backward", _Set_current_hour, _Set_current_hour, true, this, "set_time", new Object[]{Integer.valueOf(-3600)});
        _Set_current_hour.allowedEvent("time_backward", this, "set_time", new Object[]{Integer.valueOf(-3600)});
//        _Programmable_thermostat_state_machine.fires("time_backward", _Set_current_day, _Set_current_day, true, this, "set_time", new Object[]{Integer.valueOf(-86400)});
        _Set_current_day.allowedEvent("time_backward", this, "set_time", new Object[]{Integer.valueOf(-86400)});
//        _Programmable_thermostat_state_machine.fires("time_backward", _Set_program_time, _Set_program_time, true, this, "set_program_time", new Object[]{Integer.valueOf(-15)});
        _Set_program_time.allowedEvent("time_backward", this, "set_program_time", new Object[]{Integer.valueOf(-15)});
        _Programmable_thermostat_state_machine.fires("time_backward", _Current_date_and_time_refreshing, _Current_date_and_time_refreshing);
        // Each time a setup-based key is pressed, one resets the '_no_input' variable to zero so that, after 90 sec., one goes back to 'Operate':
        _Programmable_thermostat_state_machine.fires("time_backward", _Period_and_program_time_refreshing, _Period_and_program_time_refreshing, true, this, "set_no_input", new Object[]{Integer.valueOf(0)});

//        _Programmable_thermostat_state_machine.fires("time_forward", _Set_current_minute, _Set_current_minute, true, this, "set_time", new Object[]{Integer.valueOf(+60)});
        _Set_current_minute.allowedEvent("time_forward", this, "set_time", new Object[]{Integer.valueOf(+60)});
//        _Programmable_thermostat_state_machine.fires("time_forward", _Set_current_hour, _Set_current_hour, true, this, "set_time", new Object[]{Integer.valueOf(+3600)});
        _Set_current_hour.allowedEvent("time_forward", this, "set_time", new Object[]{Integer.valueOf(+3600)});
//        _Programmable_thermostat_state_machine.fires("time_forward", _Set_current_day, _Set_current_day, true, this, "set_time", new Object[]{Integer.valueOf(+86400)});
        _Set_current_day.allowedEvent("time_forward", this, "set_time", new Object[]{Integer.valueOf(+86400)});
//        _Programmable_thermostat_state_machine.fires("time_forward", _Set_program_time, _Set_program_time, true, this, "set_program_time", new Object[]{Integer.valueOf(+15)});
        _Set_program_time.allowedEvent("time_forward", this, "set_program_time", new Object[]{Integer.valueOf(+15)});
        _Programmable_thermostat_state_machine.fires("time_forward", _Current_date_and_time_refreshing, _Current_date_and_time_refreshing);
        // Each time a setup-based key is pressed, one resets the '_no_input' variable to zero so that, after 90 sec., one goes back to 'Operate':
        _Programmable_thermostat_state_machine.fires("time_forward", _Period_and_program_time_refreshing, _Period_and_program_time_refreshing, true, this, "set_no_input", new Object[]{Integer.valueOf(0)});

// Alternative: _Programmable_thermostat_state_machine.fires("time_out", _Run, _Run, !weekend(), this, "set_target_temperature",  new Object[]{Integer.valueOf(1),Integer.valueOf(4)});
//        _Programmable_thermostat_state_machine.fires("time_out", _Run, _Run, this, "not_weekend", this, "set_target_temperature", new Object[]{Integer.valueOf(1), Integer.valueOf(4)});
        _Run.allowedEvent("time_out", this, "not_weekend", this, "set_target_temperature", new Object[]{Integer.valueOf(1), Integer.valueOf(4)});
//        _Programmable_thermostat_state_machine.fires("time_out", _Run, _Run, this, "not_weekend", this, "display_target_temperature");
        _Run.allowedEvent("time_out", this, "not_weekend", this, "display_target_temperature");
// Alternative: _Programmable_thermostat_state_machine.fires("time_out", _Run, _Run, weekend(), this, "set_target_temperature",  new Object[]{Integer.valueOf(5),Integer.valueOf(8)});
//        _Programmable_thermostat_state_machine.fires("time_out", _Run, _Run, this, "weekend", this, "set_target_temperature", new Object[]{Integer.valueOf(5), Integer.valueOf(8)});
        _Run.allowedEvent("time_out", this, "weekend", this, "set_target_temperature", new Object[]{Integer.valueOf(5), Integer.valueOf(8)});
//        _Programmable_thermostat_state_machine.fires("time_out", _Run, _Run, this, "weekend", this, "display_target_temperature");
        _Run.allowedEvent("time_out", this, "weekend", this, "display_target_temperature");

        _Programmable_thermostat_state_machine.fires("time_out", _Ambient_temperature_displaying, _Current_date_and_time_displaying, this, "alternately_equal_to_two", this, "set_alternately", new Object[]{Integer.valueOf(0)});
        _Programmable_thermostat_state_machine.fires("time_out", _Ambient_temperature_displaying, _Current_date_and_time_displaying, this, "alternately_equal_to_two", this, "set_time", new Object[]{Integer.valueOf(+1)});
        /*DYNAMICAL*/ _Programmable_thermostat_state_machine.fires("time_out", _Ambient_temperature_displaying, _Ambient_temperature_displaying, this, "alternately_not_equal_to_two", this, "set_alternately", new Object[]{Integer.valueOf(_alternately + 1)});
        /*DYNAMICAL*/ _Programmable_thermostat_state_machine.fires("time_out", _Current_date_and_time_displaying, _Current_date_and_time_displaying, this, "alternately_not_equal_to_two", this, "set_alternately", new Object[]{Integer.valueOf(_alternately + 1)});

        _Programmable_thermostat_state_machine.fires("time_out", _Current_date_and_time_displaying, _Ambient_temperature_displaying, this, "alternately_equal_to_two", this, "set_alternately", new Object[]{Integer.valueOf(0)});
        _Programmable_thermostat_state_machine.fires("time_out", _Current_date_and_time_displaying, _Ambient_temperature_displaying, this, "alternately_equal_to_two", this, "set_time", new Object[]{Integer.valueOf(+1)});
        _Programmable_thermostat_state_machine.fires("time_out", _Ambient_temperature_displaying, _Ambient_temperature_displaying, this, "alternately_not_equal_to_two", this, "set_time", new Object[]{Integer.valueOf(+1)});
        _Programmable_thermostat_state_machine.fires("time_out", _Current_date_and_time_displaying, _Current_date_and_time_displaying, this, "alternately_not_equal_to_two", this, "set_time", new Object[]{Integer.valueOf(+1)});
        /*DYNAMICAL*/ _Setup.allowedEvent("time_out", this, "no_input_lessThan_ninety", this, "set_no_input", new Object[]{Integer.valueOf(_no_input + 1)});
        _Programmable_thermostat_state_machine.fires("time_out", _Setup, _Operate, this, "no_input_greaterThan_or_equal_to_ninety");

        _Programmable_thermostat_state_machine.fires("view_program", _Programmable_thermostat_state_machine, _Set_program, true, this, "set_period", new Object[]{Integer.valueOf(1)});
        // Each time a setup-based key is pressed, one resets the '_no_input' variable to zero so that, after 90 sec., one goes back to 'Operate':
        _Programmable_thermostat_state_machine.fires("view_program", _Programmable_thermostat_state_machine, _Program_refreshing, true, this, "set_no_input", new Object[]{Integer.valueOf(0)});

//        _Programmable_thermostat_state_machine.fires("view_program", _Set_period, _Set_period, _period == 8, this, "set_period", new Object[]{Integer.valueOf(1)});
        _Set_period.allowedEvent("view_program", this, "period_equal_to_eight", this, "set_period", new Object[]{Integer.valueOf(1)});
//        _Programmable_thermostat_state_machine.fires("view_program", _Set_period, _Set_period, _period != 8, this, "set_period", new Object[]{Integer.valueOf(_period + 1)});
        /*DYNAMICAL*/ _Set_period.allowedEvent("view_program", this, "period_not_equal_to_eight", this, "set_period", new Object[]{Integer.valueOf(_period + 1)});

        _air_conditioner_relay.start();
        _fan_relay.start();
        _furnace_relay.start();
        _fan_switch.start();
        _season_switch.start();
        _run_indicator.start();
        _Programmable_thermostat_state_machine.start();
    }

    public void stop() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Programmable_thermostat_state_machine.stop();
    }

    public Programmable_thermostat(Temperature temperature) throws com.pauware.pauware_engine.Exceptions.State_exception {
        init_structure(temperature);
        init_behavior();
    }

    /**
     * UML events
     */
    synchronized public void ambient_temperature_changed(Temperature temperature) throws com.pauware.pauware_engine.Exceptions.State_exception {
        /**
         * Post-condition(s)
         */
        _ambient_temperature = (Temperature) temperature.clone();
        /**
         * End of post-condition(s)
         */
        _Control.allowedEvent("ambient_temperature_changed", this, "season_switch_in_Is_cool_and_ambient_temperature_lessThan_target_temperature_minus_delta", _fan_relay, "stop", new Object[]{Boolean.valueOf(_fan_switch.in("Is auto"))});
        _Control.allowedEvent("ambient_temperature_changed", this, "season_switch_in_Is_heat_and_ambient_temperature_greaterThan_target_temperature_plus_delta", _fan_relay, "stop", new Object[]{Boolean.valueOf(_fan_switch.in("Is auto"))});
        _Programmable_thermostat_state_machine.run_to_completion("ambient_temperature_changed");
    }

    public void fan_switch_turned_on() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Programmable_thermostat_state_machine.run_to_completion("fan_switch_turned_on");
    }

    public void f_c() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Programmable_thermostat_state_machine.run_to_completion("f_c");
    }

    public void hold_temp() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Programmable_thermostat_state_machine.run_to_completion("hold_temp");
    }

    public void run_program() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Programmable_thermostat_state_machine.run_to_completion("run_program");
    }

    synchronized public void season_switch_turned_off() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Control.allowedEvent("season_switch_turned_off", _fan_relay, "stop", new Object[]{Boolean.valueOf(_fan_switch.in("Is auto"))});
        _Programmable_thermostat_state_machine.run_to_completion("season_switch_turned_off");
    }

    public void set_clock() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Programmable_thermostat_state_machine.run_to_completion("set_clock");
    }

    public void set_day() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Programmable_thermostat_state_machine.run_to_completion("set_day");
    }

    public void target_temperature_changed() throws com.pauware.pauware_engine.Exceptions.State_exception { // No use of 'synchronized' because this event is sent and consumed internally
        _Control.allowedEvent("target_temperature_changed", this, "season_switch_in_Is_cool_and_ambient_temperature_lessThan_target_temperature_minus_delta", _fan_relay, "stop", new Object[]{Boolean.valueOf(_fan_switch.in("Is auto"))});
        _Control.allowedEvent("target_temperature_changed", this, "season_switch_in_Is_heat_and_ambient_temperature_greaterThan_target_temperature_plus_delta", _fan_relay, "stop", new Object[]{Boolean.valueOf(_fan_switch.in("Is auto"))});
        _Programmable_thermostat_state_machine.run_to_completion("target_temperature_changed");
    }

    synchronized public void temp_down() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Set_program_target_temperature.allowedEvent("temp_down", this, "program_period_target_temperature_greaterThan_Min", _program[_period - 1].target_temperature, "decrement");
        _Programmable_thermostat_state_machine.run_to_completion("temp_down");
    }

    synchronized public void temp_up() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Set_program_target_temperature.allowedEvent("temp_up", this, "program_period_target_temperature_lessThan_Max", _program[_period - 1].target_temperature, "increment");
        _Programmable_thermostat_state_machine.run_to_completion("temp_up");
    }

    public void time_backward() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Programmable_thermostat_state_machine.run_to_completion("time_backward");
    }

    public void time_forward() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Programmable_thermostat_state_machine.run_to_completion("time_forward");
    }

    synchronized public void time_out(long delay, com.pauware.pauware_engine.Core.AbstractState context) throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Programmable_thermostat_state_machine.fires("time_out", _Ambient_temperature_displaying, _Ambient_temperature_displaying, this, "alternately_not_equal_to_two", this, "set_alternately", new Object[]{Integer.valueOf(_alternately + 1)});
        _Programmable_thermostat_state_machine.fires("time_out", _Current_date_and_time_displaying, _Current_date_and_time_displaying, this, "alternately_not_equal_to_two", this, "set_alternately", new Object[]{Integer.valueOf(_alternately + 1)});
        _Setup.allowedEvent("time_out", this, "no_input_lessThan_ninety", this, "set_no_input", new Object[]{Integer.valueOf(_no_input + 1)});
        _Programmable_thermostat_state_machine.run_to_completion("time_out");
    }

    public void time_out_error(com.pauware.pauware_engine.Exceptions.State_exception se) throws com.pauware.pauware_engine.Exceptions.State_exception {
        se.printStackTrace();
        // possible fault recovery here...
    }

    synchronized public void view_program() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _Set_period.allowedEvent("view_program", this, "period_not_equal_to_eight", this, "set_period", new Object[]{Integer.valueOf(_period + 1)});
        _Programmable_thermostat_state_machine.run_to_completion("view_program");
    }

    /**
     * UML actions
     */
    public void display_ambient_temperature() {
        if (_user_interface != null) {
            switch (_temperature_mode) {
                case Temperature.Celsius:
                    _user_interface.display_ambient_temperature(_ambient_temperature.asStringCelsius(1), _ambient_temperature);
                    break;
                case Temperature.Fahrenheit:
                    _user_interface.display_ambient_temperature(_ambient_temperature.asStringFahrenheit(1), _ambient_temperature);
                    break;
                default:
                    _user_interface.display_ambient_temperature(String.valueOf(null), _ambient_temperature);
            }
        }
    }

    public void display_current_date_and_time() {
        if (_user_interface != null) {
            _user_interface.display_current_date_and_time(_time.getTime().toString());
        }
    }

    public void display_period_and_program_time() {
        if (_user_interface != null) {
            _user_interface.display_period_and_program_time(String.valueOf(_period) + " - " + _program[_period - 1].time.getTime());
        }
    }

    public void display_program_target_temperature() {
        if (_user_interface != null) {
            _user_interface.display_program_target_temperature(_program[_period - 1].target_temperature, _temperature_mode);
        }
    }

    public void display_target_temperature() {
        if (_user_interface != null) {
            _user_interface.display_target_temperature(_target_temperature, _temperature_mode);
        }
    }

    public void program_at_period_target_temperature_decrement(int index) {
        _program[index].target_temperature.decrement();
    }

    public void program_at_period_target_temperature_increment(int index) {
        _program[index].target_temperature.increment();
    }

    public void set_alternately(Integer i) {
        _alternately = i.intValue();
    }

    public void set_no_input(Integer i) {
        _no_input = i.intValue();
    }

    public void set_period(Integer i) {
        _period = i.intValue();
    }

    public void set_program_time(Integer step) {
        java.util.Date date = _program[_period - 1].time.getTime();
        date.setTime(date.getTime() + step.intValue() * 1000);
        _program[_period - 1].time.setTime(date);
    }

    private int _index_of_last_target_temperature = -1;

    public void set_target_temperature(Integer min, Integer max) {
        long time = _time.get(java.util.Calendar.HOUR_OF_DAY) * 3600L + _time.get(java.util.Calendar.MINUTE) * 60L + _time.get(java.util.Calendar.SECOND);
        for (int i = min.intValue() - 1; i < max.intValue(); i++) {
            _program[i].difference = time - (_program[i].time.get(java.util.Calendar.HOUR_OF_DAY) * 3600L + _program[i].time.get(java.util.Calendar.MINUTE) * 60L + _program[i].time.get(java.util.Calendar.SECOND));
        }
        long difference = _program[min.intValue() - 1].difference;
        int index = min.intValue() - 1;
        for (int i = min.intValue(); i < max.intValue(); i++) {
            if (_program[i].difference >= 0 && _program[i].difference <= difference) {
                difference = _program[i].difference;
                index = i;
            }
        }
        if (index != _index_of_last_target_temperature) {
            try {
                _target_temperature = (Temperature) _program[index].target_temperature.clone();
                _index_of_last_target_temperature = index;
            } // Java ME problem:
            // catch(CloneNotSupportedException cnse) {/* Cannot occur whether 'class Temperature implements Cloneable' -> Java ME */}
            catch (Exception e) {
            }
        }
    }

    public void set_time(Integer step) {
        java.util.Date date = _time.getTime();
        date.setTime(date.getTime() + step.intValue() * 1000);
        _time.setTime(date);
    }

    public void switch_mode() {
        switch (_temperature_mode) {
            case Temperature.Celsius:
                _temperature_mode = Temperature.Fahrenheit;
                break;
            case Temperature.Fahrenheit:
                _temperature_mode = Temperature.Celsius;
                break;
            default:
                _temperature_mode = Temperature.Fahrenheit;
        }
    }

    /**
     * UML guards
     */
    public boolean alternately_equal_to_two() {
        return _alternately == 2;
    }

    public boolean alternately_not_equal_to_two() {
        return _alternately != 2;
    }

    public boolean no_input_lessThan_ninety() {
        return _no_input < From_Setup_to_Operate_delay;
    }

    public boolean no_input_greaterThan_or_equal_to_ninety() {
        return _no_input >= From_Setup_to_Operate_delay;
    }

    public boolean program_period_target_temperature_greaterThan_Min() {
        return _program[_period - 1].target_temperature.greaterThan(_Min);
    }

    public boolean program_period_target_temperature_lessThan_Max() {
        return _program[_period - 1].target_temperature.lessThan(_Max);
    }

    public boolean period_equal_to_eight() {
        return _period == 8;
    }

    public boolean period_not_equal_to_eight() {
        return _period != 8;
    }

    public boolean season_switch_in_Is_cool_and_ambient_temperature_greaterThan_target_temperature() {
        return _season_switch.in("Is cool") && _ambient_temperature.greaterThan(_target_temperature);
    }

    public boolean season_switch_in_Is_cool_and_ambient_temperature_lessThan_target_temperature_minus_delta() {
        return _season_switch.in("Is cool") && _ambient_temperature.asCelsius() < (_target_temperature.asCelsius() - _delta);
    }

    public boolean season_switch_in_Is_heat_and_ambient_temperature_lessThan_target_temperature() {
        return _season_switch.in("Is heat") && _ambient_temperature.lessThan(_target_temperature);
    }

    public boolean season_switch_in_Is_heat_and_ambient_temperature_greaterThan_target_temperature_plus_delta() {
        return _season_switch.in("Is heat") && _ambient_temperature.asCelsius() > (_target_temperature.asCelsius() + _delta);
    }

    public boolean target_temperature_greaterThan_Min() {
        return _target_temperature.greaterThan(_Min);
    }

    public boolean target_temperature_lessThan_Max() {
        return _target_temperature.lessThan(_Max);
    }

    public boolean not_weekend() {
        return !weekend();
    }

    public boolean weekend() {
        return _time.get(java.util.Calendar.DAY_OF_WEEK) == java.util.Calendar.SATURDAY || _time.get(java.util.Calendar.DAY_OF_WEEK) == java.util.Calendar.SUNDAY;
    }

    /**
     * Implementation
     */
    public void fan_switch_auto() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _fan_switch.auto();
    }

    public void fan_switch_on() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _fan_switch.on();
    }

    public void season_switch_cool() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _season_switch.cool();
    }

    public void season_switch_heat() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _season_switch.heat();
    }

    public void season_switch_off() throws com.pauware.pauware_engine.Exceptions.State_exception {
        _season_switch.off();
    }

    /**
     * JavaBeans component model not supported by Java ME:
     */
    public void propertyChange(final /*java.beans.PropertyChangeEvent*/ String event) {
        // if(_user_interface != null && event.getPropertyName().equals(Run_indicator.Status)) _user_interface.update_run_indicator_status((String)event.getNewValue());
        if (_user_interface != null) {
            _user_interface.update_run_indicator_status(event);
        }
    }

    /**
     * GUI utilities
     */
    public boolean furnace_relay_is_on() {
        return _furnace_relay.is_on();
    }

    public boolean air_conditioner_relay_is_on() {
        return _air_conditioner_relay.is_on();
    }

    /**
     * Management
     */
    public String async_current_state() {
        return _Programmable_thermostat_state_machine.async_current_state();
    }

    public String current_state() {
        return _Programmable_thermostat_state_machine.current_state();
    }

    public boolean in_state(String name) {
        return _Programmable_thermostat_state_machine.in_state(name);
    }

    public String name() {
        return _Programmable_thermostat_state_machine.name();
    }

    public String verbose() {
        return _Programmable_thermostat_state_machine.verbose();
    }
}
